@@ -5,8 +5,8 @@ use base 'PDF::Builder::Basic::PDF::Dict';
 use strict;
 no warnings qw[ recursion uninitialized ];
 
-our $VERSION = '3.013'; # VERSION
-my $LAST_UPDATE = '3.010'; # manually update whenever code is changed
+our $VERSION = '3.014'; # VERSION
+my $LAST_UPDATE = '3.014'; # manually update whenever code is changed
 
 use Carp;
 use Encode qw(:all);
@@ -18,18 +18,26 @@ use PDF::Builder::Basic::PDF::Utils;
 
 our $cmap = {};
 
+# for new() if not using find_ms() or .cmap files
+# may be overridden fully or partially by -cmaps option
+# [0] is Windows list, [1] is non-Windows list  Platform/Encoding
+# can substitute 'find_ms' instead of a list of P/E
+# suggested default list by Alfred Reibenschuh (original PDF::API2 author)
+my @default_CMap = ('0/6 3/10 0/4 3/1 0/3', '0/6 0/4 3/10 0/3 3/1');
+
 =head1 NAME
 
 PDF::Builder::Resource::CIDFont::TrueType::FontFile - additional code support for TT font files. Inherits from L<PDF::Builder::Basic::PDF::Dict>
 
 =cut
 
+# identical routine in Resource/CIDFont/CJKFont.pm
 sub _look_for_cmap {
     my $fname = lc(shift);
 
     $fname =~ s/[^a-z0-9]+//gi;
     return ({%{$cmap->{$fname}}}) if defined $cmap->{$fname};
-    eval "require 'PDF/Builde/Resource/CIDFont/CMap/$fname.cmap'"; ## no critic
+    eval "require 'PDF/Builder/Resource/CIDFont/CMap/$fname.cmap'"; ## no critic
     unless ($@) {
         return {%{$cmap->{$fname}}};
     } else {
@@ -313,6 +321,12 @@ sub new {
     my ($class, $pdf, $file, %opts) = @_;
 
     my $data = {};
+# some debug settings
+#$opts{'-debug'} = 1;
+#$opts{'-cmaps'} = '0/6, 0/4, 3/10, 0/3, 3/1';
+#$opts{'-cmaps'} = '7/8; 8/7';  # invalid P/E, should use find_ms instead
+#$opts{'-cmaps'} = 'find_ms;   find_ms ';
+#$opts{'-usecmf'} = 1;
 
     confess "cannot find font '$file'" unless -f $file;
     my $font = Font::TTF::Font->open($file);
@@ -426,6 +440,20 @@ sub new {
         $data->{'cff'} = readcffstructs($font);
     }
 
+    if ($opts{'-debug'}) {
+        print "CMap determination for file $file\n";
+    }
+    if ($data->{'issymbol'}) {
+	# force 'find_ms' if we know it's a symbol font anyway
+	if ($opts{'-debug'}) {
+            print "This is a symbol font 3/0\n";
+	}
+	$opts{'-cmaps'} = 'find_ms';
+    }
+
+    # first, see if CJK .cmap file exists, and want to use it
+    # apparently, very old CJK fonts lack internal cmap tables and need this
+    my $CMapfile = '';
     if (defined $data->{'cff'}->{'ROS'}) {
         my %cffcmap = (
             'Adobe:Japan1' => 'japanese',
@@ -433,20 +461,178 @@ sub new {
             'Adobe:CNS1'   => 'traditional',
             'Adobe:GB1'    => 'simplified',
         );
-        my $ccmap = _look_for_cmap($cffcmap{"$data->{'cff'}->{'ROS'}->[0]:$data->{'cff'}->{'ROS'}->[1]"});
+        $CMapfile = $cffcmap{"$data->{'cff'}->{'ROS'}->[0]:$data->{'cff'}->{'ROS'}->[1]"};
+	if ($opts{'-debug'}) {
+	    if ($CMapfile ne '') {
+		print "Available CMap file $CMapfile.cmap\n";
+	    } else {
+		print "No CMap file found\n";
+	    }
+	}
+    }
+    my $CMap = $CMapfile;  # save original name for later
+    if ($CMapfile ne '' && $opts{'-usecmf'}) {
+        my $ccmap = _look_for_cmap($CMapfile);
         $data->{'u2g'} = $ccmap->{'u2g'};
         $data->{'g2u'} = $ccmap->{'g2u'};
     } else {
+	# there is no .cmap file for this alphabet, or we don't want to use it
+	if ($opts{'-debug'} && $CMapfile ne '') {
+	    print "Choose not to use .cmap file\n";
+	}
         $data->{'u2g'} = {};
 
-        my $gmap = $font->{'cmap'}->read()->find_ms()->{'val'};
-        foreach my $u (sort {$a <=> $b} keys %{$gmap}) {
-            my $uni = $u || 0;
-            $data->{'u2g'}->{$uni} = $gmap->{$uni};
-        }
-        $data->{'g2u'} = [ map { $_ || 0 } $font->{'cmap'}->read()->reverse() ];
-    }
+	if ($opts{'-debug'}) {
+            # debug stuff
+            my $numTables = $font->{'cmap'}{'Num'}; # number of subtables in cmap table
+            for my $iii (0 .. $numTables-1) {
+              print "CMap Table $iii, ";
+              print " Platform/Encoding = ";
+	      print   $font->{'cmap'}{'Tables'}[$iii]{'Platform'};
+	      print   "/";
+              print   $font->{'cmap'}{'Tables'}[$iii]{'Encoding'};
+              print ", Format = ".$font->{'cmap'}{'Tables'}[$iii]{'Format'};
+              print ", Ver = ".$font->{'cmap'}{'Tables'}[$iii]{'Ver'};
+              print "\n";
+            }
+	}
 
+        # Platform
+	#   0 = Unicode
+	#   1 = Mac (deprecated)
+	#   2 = ISO (deprecated in favor of Unicode)
+	#   3 = Windows
+	#   4 = Custom
+	# Encodings 
+	#   Platform 0 (Unicode)
+	#     0 = Unicode 1.0
+	#     1 = Unicode 1.1
+	#     2 = ISO/IEC 10646
+	#     3 = Unicode 2.0+ BMP only, formats 0/4/6
+	#     4 = Unicode 2.0+ full repertoire, formats 0/4/6/10/12
+	#     5 = Unicode Variation Sequences, format 14
+	#     6 = Unicode full repertoire, formats 0/4/6/10/12/13
+	#   Platform 1 (Macintosh) has encodings 0-32 for various alphabets
+	#   Platform 2 (ISO)
+	#     0 = 7 bit ASCII
+	#     1 = ISO 10646
+	#     2 = ISO 8859-1
+	#   Platform 3 (Windows)
+	#     0 = Symbol
+	#     1 = Unicode BMP
+	#     2 = ShiftJIS
+	#     3 = PRC
+	#     4 = Big5
+	#     5 = Wansung
+	#     6 = Johab
+	#     7-9 = Reserved
+	#     10 = Unicode full repertoire
+	#   Platform 4 (Custom)
+	#     0-255 OTF Windows NT compatibility mapping
+	# Format 0-14 ?
+	# Ver ?
+
+	my $cmap_list = '';
+	my $OS = $^O;
+	if ($opts{'-debug'}) {
+	    print "OS string is '$OS', ";
+	}
+	if ($OS eq 'MSWin32' || $OS eq 'dos' || 
+	    $OS eq 'os2' || $OS eq 'cygwin') {
+	    $OS = 0; # Windows request
+	    if ($opts{'-debug'}) {
+	        print "treat as Windows platform\n";
+	    }
+	} else {
+	    $OS = 1; # non-Windows request
+	    if ($opts{'-debug'}) {
+	        print "treat as non-Windows platform\n";
+	    }
+	}
+	my $gmap;
+	if (defined $opts{'-cmaps'}) {
+	    $CMap = $opts{'-cmaps'};
+	    # 1 or 2 lists, Windows and non-Windows, separated by ;
+	    # if no ;, assume same list applies to both Platforms
+	    # a list may be the string 'find_ms' to just use that mode
+	    # otherwise, a list is p1/e1 p2/e2 etc. separated by max 1 comma
+	    #   and any number of whitespace
+	    if (index($CMap, ';') < 0) {
+		# no ;, so single entry for both
+		$CMap = $CMap.";".$CMap;
+	    }
+	    $cmap_list = (split /;/, $CMap)[$OS];
+	    $cmap_list =~ s/^\s+//;
+	    $cmap_list =~ s/\s+$//;
+	} else {
+	    # will use @default_CMap list
+	    $cmap_list = '';
+	}
+	if ($cmap_list eq '') {
+	    # empty list? use default CMap entry
+	    $cmap_list = $default_CMap[$OS];
+	}
+	# now we have a cmap_list string of target P/E's to look for (either
+	# specified with -cmap, or default), OR just 'find_ms'
+	if ($opts{'-debug'}) {
+	    print "search list '$cmap_list' for match, else find_ms()\n";
+	}
+	if ($cmap_list eq 'find_ms') {
+	    # use original find_ms() call
+            $gmap = $font->{'cmap'}->read()->find_ms();
+	} else {
+	    my @list = split/[,\s]+/, $cmap_list;
+	    # should be list of P/E settings, like 0/6, 3/10, etc.
+	    # following after code from Bob Hallissy (TTF::Font author)
+	    my ($cmap, %cmaps, $i);
+	    $cmap = $font->{'cmap'}->read();
+	    for ($i = 0; $i < $font->{'cmap'}{'Num'}; $i++) {
+		my $s = $font->{'cmap'}{'Tables'}[$i];
+		my $key = "$s->{'Platform'}/$s->{'Encoding'}";
+		$cmaps{$key} = $s;
+	    }
+	    foreach (@list) {
+		if ($_ eq '') { next; } # empty entry got into list?
+		if (exists $cmaps{$_}) {
+		    $cmap->{' mstable'} = $cmaps{$_}; # might be unnecessary
+		    if ($opts{'-debug'}) {
+			print "found internal cmap table '$_' on search list\n";
+		    }
+		    $gmap = $cmaps{$_};
+		    last;
+		}
+	    }
+	} # not 'find_ms' request
+
+	# final check (.cmap wasn't used). no useful internal cmap found?
+	if (! $gmap) {
+	    # ignored existing .cmap before? use it anyway
+            if ($CMapfile ne '' && !$opts{'-usecmf'}) {
+		if ($opts{'-debug'}) {
+		    print "need to use .cmap file '$CMapfile.cmap' anyway\n";
+		}
+                my $ccmap = _look_for_cmap($CMapfile);
+                $data->{'u2g'} = $ccmap->{'u2g'};
+                $data->{'g2u'} = $ccmap->{'g2u'};
+            } else {
+		# Hail Mary pass to use find_ms()
+                $gmap = $font->{'cmap'}->read()->find_ms();
+		if (! $gmap) {
+	            die "No useful internal cmap found for $file\n";
+		}
+	    }
+	} 
+	# we SHOULD have a valid $gmap at this point
+	# load up data->u2g and g2u from gmap (one 'Tables' entry)
+        $gmap = $gmap->{'val'};
+	foreach my $u (sort {$a<=>$b} keys %{$gmap}) {
+	    my $uni = $u || 0;
+	    $data->{'u2g'}->{$uni} = $gmap->{$uni};
+	}
+	$data->{'g2u'} = [ map { $_ || 0 } $font->{'cmap'}->read()->reverse() ];
+    } # no .cmap or don't want to use it
+
+    # 3/0 cmap table
     if ($data->{'issymbol'}) {
         map { $data->{'u2g'}->{$_} ||= $font->{'cmap'}->read()->ms_lookup($_) } (0xf000 .. 0xf0ff);
         map { $data->{'u2g'}->{$_ & 0xff} ||= $font->{'cmap'}->read()->ms_lookup($_) } (0xf000 .. 0xf0ff);
